@@ -101,8 +101,15 @@ findGlobals <- function(expr, envir=parent.frame(), ..., tweak=NULL, dotdotdot=c
 
   if (substitute) expr <- substitute(expr)
 
+  mdebug("findGlobals(..., dotdotdot = '%s', method = '%s', unlist = %s) ...", dotdotdot, method, unlist)
+  
   if (is.list(expr)) {
+    mdebug(" - expr: <a list of length %d>", length(expr))
+    
     globals <- lapply(expr, FUN=findGlobals, envir=envir, ..., tweak=tweak, dotdotdot=dotdotdot, substitute=FALSE, unlist=FALSE)
+
+    mdebug(" - preliminary globals found: [%d] %s", length(globals), hpaste(sQuote(names(globals))))
+    
     if (unlist) {
       needsDotdotdot <- FALSE
       for (kk in seq_along(globals)) {
@@ -118,10 +125,17 @@ findGlobals <- function(expr, envir=parent.frame(), ..., tweak=NULL, dotdotdot=c
       globals <- sort(unique(globals))
       if (needsDotdotdot) globals <- c(globals, "...")
     }
+
+    mdebug(" - globals found: [%d] %s", length(globals), hpaste(sQuote(globals)))
+    
+    mdebug("findGlobals(..., dotdotdot = '%s', method = '%s', unlist = %s) ... DONE", dotdotdot, method, unlist)
     return(globals)
   }
 
-  if (is.function(tweak)) expr <- tweak(expr)
+  if (is.function(tweak)) {
+    mdebug(" - tweaking expression using function")
+    expr <- tweak(expr)
+  }
 
   if (method == "ordered") {
     findGlobalsT <- findGlobals_ordered
@@ -139,8 +153,16 @@ findGlobals <- function(expr, envir=parent.frame(), ..., tweak=NULL, dotdotdot=c
     findGlobalsT(expr, envir=envir)
   }, warning=function(w) {
     ## Warned about '...'?
+    ## NOTE: The warning we're looking for is the one generated by
+    ## codetools::findGlobals().  That warning is _not_ translated,
+    ## meaning this approach should work as is as long as the message
+    ## is not modified by codetools itself.  If codetools ever changes
+    ## this such that the below string matching fails, then the package
+    ## tests (tests/dotdotdot.R) will detect that.  In other words,
+    ## such a change will not go unnoticed.  /HB 2017-03-08
     pattern <- "... may be used in an incorrect context"
     if (grepl(pattern, w$message, fixed=TRUE)) {
+      mdebug(" - detected: %s", dQuote(trim(w$message)))
       needsDotdotdot <<- TRUE
       if (dotdotdot == "return") {
         ## Consume / muffle warning
@@ -158,6 +180,10 @@ findGlobals <- function(expr, envir=parent.frame(), ..., tweak=NULL, dotdotdot=c
 
   if (needsDotdotdot) globals <- c(globals, "...")
 
+  mdebug(" - globals found: [%d] %s", length(globals), hpaste(sQuote(globals)))
+  
+  mdebug("findGlobals(..., dotdotdot = '%s', method = '%s', unlist = %s) ... DONE", dotdotdot, method, unlist)
+  
   globals
 }
 
