@@ -63,9 +63,12 @@ globalsOf <- function(expr, envir=parent.frame(), ..., method=c("ordered", "cons
 
   if (substitute) expr <- substitute(expr)
 
+  mdebug("globalsOf(..., method = '%s', mustExist = %s, unlist = %s, recursive = %s) ...", method, mustExist, unlist, recursive)
+  
   ## 1. Identify global variables (static code inspection)
   names <- findGlobals(expr, envir=envir, ..., method=method, tweak=tweak, substitute=FALSE, unlist=unlist)
-
+  mdebug(" - preliminary globals (by name): [%d] %s", length(names), hpaste(sQuote(names)))
+  
   ## 2. Locate them (run time)
   globals <- tryCatch({
     globalsByName(names, envir=envir, mustExist=mustExist)
@@ -77,28 +80,50 @@ globalsOf <- function(expr, envir=parent.frame(), ..., method=c("ordered", "cons
     stop(ex)
   })
 
+  mdebug(" - preliminary globals (by value): [%d] %s", length(globals), hpaste(sQuote(names(globals))))
+  
   ## 3. Among globals that are closures (functions) and that exist outside
   ##    of namespaces ("packages"), check for additional globals?
   if (recursive) {
+    mdebug(" - recursive scan of preliminary globals ...")
+
     ## Don't enter functions in namespaces / packages
-    where <- sapply(globals, FUN = function(x) environmentName(environment(x)))
+    where <- attr(globals, "where")
+    stopifnot(length(where) == length(globals))
+    where <- sapply(where, FUN = envname)
     globalsT <- globals[!(where %in% loadedNamespaces())]
 
+    mdebug(" - subset of globals to be scanned (not in loaded namespaces): [%d] %s", length(globalsT), hpaste(sQuote(names(globalsT))))
+    
     ## Enter only functions
-    globalsT <- globals[sapply(globalsT, FUN = typeof) == "closure"]
+    ## NOTE: This excludes functions "not found", but also primitives not dropped above.
+    globalsT <- globalsT[sapply(globalsT, FUN = typeof) == "closure"]
         
     if (length(globalsT) > 0) {
+      mdebug(" - subset of globals to be scanned: [%d] %s", length(globalsT), hpaste(sQuote(names(globalsT))))
+      namesT <- names(globalsT)
       for (gg in seq_along(globalsT)) {
+        mdebug("   + scanning global #%d (%s) ...", gg, sQuote(namesT[[gg]]))
         fcn <- globalsT[[gg]]
-	globalsGG <- globalsOf(fcn, envir=envir, ..., method=method, tweak=tweak, substitute=FALSE, mustExist=mustExist, unlist=unlist, recursive=recursive)
+        env <- environment(fcn) ## was 'env <- envir' in globals 0.8.0.
+	globalsGG <- globalsOf(fcn, envir=env, ..., method=method, tweak=tweak, substitute=FALSE, mustExist=mustExist, unlist=unlist, recursive=recursive)
 	if (length(globalsGG) > 0) {
 	  globals <- c(globals, globalsGG)
 	}
       }
       globals <- unique(globals)
+      mdebug(" - updated set of globals found: [%d] %s", length(globals), hpaste(sQuote(names(globals))))
+    } else {
+      mdebug(" - subset of globals to be scanned: [0]")
     }
+    
+    mdebug(" - recursive scan of preliminary globals ... DONE")
   }
 
+  mdebug(" - globals found: [%d] %s", length(globals), hpaste(sQuote(names(globals))))
+  
+  mdebug("globalsOf(..., method = '%s', mustExist = %s, unlist = %s, recursive = %s) ... DONE", method, mustExist, unlist, recursive)
+  
   globals
 } ## globalsOf()
 
@@ -120,15 +145,22 @@ globalsOf <- function(expr, envir=parent.frame(), ..., method=c("ordered", "cons
 globalsByName <- function(names, envir=parent.frame(), mustExist=TRUE, ...) {
   names <- as.character(names)
 
+  mdebug("globalsByName(<%d names>, mustExist = %s) ...", length(names), mustExist)
+  mdebug("- search from environment: %s", sQuote(envname(envir)))
+
   ## Locate and retrieve the specified globals
   n <- length(names)
   needsDotdotdot <- (identical(names[n], "..."))
   if (needsDotdotdot) names <- names[-n]
-
+  mdebug("- dotdotdot: %s", needsDotdotdot)
+  
   globals <- structure(list(), class=c("Globals", "list"))
   where <- list()
-  for (name in names) {
+  for (kk in seq_along(names)) {
+    name <- names[kk]
+    mdebug("- locating #%d (%s)", kk, sQuote(name))
     env <- where(name, envir=envir, inherits=TRUE)
+    mdebug("  + found in environment: %s", sQuote(envname(env)))
     if (!is.null(env)) {
       where[[name]] <- env
       value <- get(name, envir=env, inherits=FALSE)
@@ -166,5 +198,7 @@ globalsByName <- function(names, envir=parent.frame(), mustExist=TRUE, ...) {
 
   attr(globals, "where") <- where
 
+  mdebug("globalsByName(<%d names>, mustExist = %s) ... DONE", length(names), mustExist)
+  
   globals
 } ## globalsByName()
